/*
 * ePeriodique: an EFL Periodic Table Of Elements
 * Copyright (C) 2010-2012 Jerome Pinot <ngc891@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "eperiodique.h"
#include "data.h"

Evas_Object *window, *flip;
char edje_file[PATH_MAX];

void about_del(void *data, Evas *e, Evas_Object *obj, void *event_info);
void popup_about(void *data, Evas *e, Evas_Object *obj, void *event_info);
void do_flip(void *data, Evas *e, Evas_Object *obj, void *event_info);
void get_data(void *data, Evas *e, Evas_Object *obj, void *event_info);
void show_data(void *data, Evas *e, Evas_Object *obj, void *event_info);
int create_panel(Evas_Object *window, Evas_Object *table);

static void
_quit(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
{
   elm_exit();
}

static const Ecore_Getopt optdesc = {
  "eperiodique",
  "%prog [options]",
  PACKAGE_VERSION,
  COPYRIGHT,
  "BSD with advertisement clause",
  "An EFL periodic table of the elements",
  0,
  {
    ECORE_GETOPT_STORE_TRUE('f', "fullscreen", "make the application fullscreen"),
    ECORE_GETOPT_STORE_STR('t', "theme", "change theme"),
    ECORE_GETOPT_LICENSE('L', "license"),
    ECORE_GETOPT_COPYRIGHT('C', "copyright"),
    ECORE_GETOPT_VERSION('V', "version"),
    ECORE_GETOPT_HELP('h', "help"),
    ECORE_GETOPT_SENTINEL
  }
};

EAPI_MAIN int
elm_main(int argc, char **argv)
{
   int xpos, ypos;
   int args, i;
   char *theme = "default";
   char string[8];
   Evas_Object *background, *table, *button, *icon, *cell;
   void *address = NULL;
   int width, height;
   Eina_Bool fullscreen = EINA_FALSE;
   Eina_Bool quit_option = EINA_FALSE;
   Ecore_Getopt_Value values[] = {
     ECORE_GETOPT_VALUE_BOOL(fullscreen),
     ECORE_GETOPT_VALUE_STR(theme),
     ECORE_GETOPT_VALUE_BOOL(quit_option),
     ECORE_GETOPT_VALUE_BOOL(quit_option),
     ECORE_GETOPT_VALUE_BOOL(quit_option),
     ECORE_GETOPT_VALUE_BOOL(quit_option),
     ECORE_GETOPT_VALUE_NONE
   };

#if ENABLE_NLS
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   bind_textdomain_codeset(PACKAGE, "UTF-8");
   textdomain(PACKAGE);
#endif

   /* Parse command line options */
   args = ecore_getopt_parse(&optdesc, values, argc, argv);
   if (args < 0)
     {
       EINA_LOG_CRIT("Could not parse arguments.");
       goto end;
     }
   else if (quit_option)
     {
       goto end;
     }

   /* Set general properties */
   window = elm_win_add(NULL, PACKAGE, ELM_WIN_BASIC);
   elm_win_title_set(window, "ePeriodique");
   elm_win_autodel_set(window, EINA_TRUE);
   elm_policy_set(ELM_POLICY_QUIT, ELM_POLICY_QUIT_LAST_WINDOW_CLOSED);


   /* Add a background */
   background = elm_bg_add(window);
   evas_object_size_hint_weight_set(background, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(background, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_win_resize_object_add(window, background);
   evas_object_show(background);

   /* Add the main table for storing elements */
   table = elm_table_add(window);
   elm_table_homogeneous_set(table, EINA_TRUE);
   elm_win_resize_object_add(window, table);
   evas_object_size_hint_weight_set(table, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(table, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(table);

   /* Get the edje theme file */
   snprintf(edje_file, sizeof(edje_file), "%s/themes/%s.edj", PACKAGE_DATA_DIR, theme);
   if (access(edje_file, R_OK) != 0)
     {
        EINA_LOG_CRIT("Loading theme error: can not read %s", edje_file);
        return EXIT_FAILURE;
     }

   /* Fill group numbers */
   for (i = 1; i < 19; i++)
     {
        cell = elm_layout_add(window);
        elm_layout_file_set(cell, edje_file, "Number-box");
        evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
        snprintf(string, sizeof(string), "%d", i);
        elm_object_part_text_set(cell, "Number", string);
        elm_object_signal_emit(cell, "arrow-down", "");
        if ( i == 1 || i == 18 )
          {
             elm_table_pack(table, cell, i, 0, 1, 1);
          }
        else if ( i > 2 && i < 13 )
          {
             elm_table_pack(table, cell, i, 3, 1, 1);
          }
        else
          {
             elm_table_pack(table, cell, i, 1, 1, 1);
          }
        elm_object_tooltip_text_set(cell, _("group"));
        evas_object_show(cell);
     }

   /* Fill period numbers */
   for (i = 1; i < 8; i++)
     {
        cell = elm_layout_add(window);
        elm_layout_file_set(cell, edje_file, "Number-box");
        evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
        snprintf(string, sizeof(string), "%d", i);
        elm_object_part_text_set(cell, "Number", string);
        elm_object_signal_emit(cell, "arrow-right", "");
        elm_table_pack(table, cell, 0, i, 1, 1);
        elm_object_tooltip_text_set(cell, _("period"));
        evas_object_show(cell);
     }

   /* Fill the elements */
   for (i = 0; i < ZMAX; i++)
     {
        /* Get element position */
        xpos = elements_data[i].xpos + 1;
        ypos = elements_data[i].ypos + 1;

        /* Get box properties from theme */
        cell = elm_layout_add(window);
        elm_layout_file_set(cell, edje_file, "Element-box");
        evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);

        /* Change element background according to group colour */
        elm_object_signal_emit(cell, elements_data[i].category, "");

        /* Put in the text data */
        snprintf(string, sizeof(string), "%d", elements_data[i].atomic_number);
        elm_object_part_text_set(cell, "Z", string);
        elm_object_part_text_set(cell, "Symbol", elements_data[i].symbol);
        snprintf(string, sizeof(string), "%'g", elements_data[i].atomic_weight);
        elm_object_part_text_set(cell, "Mass", string);

        /* Change symbol colour according to phase */
        elm_object_signal_emit(cell, elements_data[i].phase, "");

        /* We assign a callback to each cell with i as data to recognize each element */
        memcpy(&address, &i, sizeof(i));
        evas_object_event_callback_add(cell, EVAS_CALLBACK_MOUSE_DOWN, show_data, address);

        /* Finally add a tooltip */
        elm_object_tooltip_text_set(cell, _(elements_data[i].name));

        /* Pack the element cell in the right position in table */
        elm_table_pack(table, cell, xpos, ypos, 1, 1);
        evas_object_show(cell);
     }

   /* Add the asterisks */
   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Element-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_object_signal_emit(cell, "lanthanides", "");
   elm_object_part_text_set(cell, "Symbol", "*");
   elm_table_pack(table, cell, 3, 6, 1, 1);
   elm_object_tooltip_text_set(cell, _("lanthanides"));
   evas_object_show(cell);

   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Number-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_object_part_text_set(cell, "Number", "*");
   elm_table_pack(table, cell, 2, 9, 1, 1);
   elm_object_tooltip_text_set(cell, _("lanthanides"));
   evas_object_show(cell);

   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Element-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_object_signal_emit(cell, "actinides", "");
   elm_object_part_text_set(cell, "Symbol", "**");
   elm_table_pack(table, cell, 3, 7, 1, 1);
   elm_object_tooltip_text_set(cell, _("actinides"));
   evas_object_show(cell);

   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Number-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_object_part_text_set(cell, "Number", "**");
   elm_table_pack(table, cell, 2, 10, 1, 1);
   elm_object_tooltip_text_set(cell, _("actinides"));
   evas_object_show(cell);

   /* Add title */
   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Title-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_object_part_text_set(cell, "Title", "ePeriodique");
   elm_table_pack(table, cell, 5, 1, 6, 1);
   evas_object_event_callback_add(cell, EVAS_CALLBACK_MOUSE_DOWN, popup_about, window);
   evas_object_show(cell);

   /* Add a blank cell on the bottom right to get a right/bottom margin */
   cell = elm_layout_add(window);
   elm_layout_file_set(cell, edje_file, "Number-box");
   evas_object_size_hint_weight_set(cell, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(cell, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_table_pack(table, cell, 19, 11, 1, 1);
   evas_object_show(cell);

   /* Add a panel for molar mass computation */
   if (create_panel(window, table) != EXIT_SUCCESS)
     EINA_LOG_ERR("Can't create the panel!");

   /* button to close, useful on mobile device */
   button = elm_button_add(window);
   icon = elm_icon_add(window);
   elm_icon_standard_set(icon, "close");
   elm_object_part_content_set(button, "icon", icon);
   evas_object_smart_callback_add(button, "clicked", _quit, NULL);
   elm_table_pack(table, button, 19, 0, 1, 1);
   evas_object_size_hint_align_set(button, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(button);

   /* Fill the window with the flip object */
   flip = elm_flip_add(window);
   evas_object_size_hint_weight_set(flip, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(flip, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_win_resize_object_add(window, flip);
   elm_flip_interaction_set(flip, ELM_FLIP_INTERACTION_NONE);
   elm_object_part_content_set(flip, "front", table);
   evas_object_show(flip);

   if (fullscreen == EINA_TRUE)
     {
        Evas_Object *conform;
        elm_win_conformant_set(window, EINA_TRUE);
        conform = elm_conformant_add(window);
        elm_win_resize_object_add(window, conform);
        evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_show(conform);
        elm_object_content_set(conform, flip);
     }
   else
     {
        /* Get window's size from edje and resize it */
        width = atoi(edje_file_data_get(edje_file, "width"));
        height = atoi(edje_file_data_get(edje_file, "height"));
        evas_object_resize(window, width, height);
     }

   elm_object_focus_set(window, EINA_TRUE);
   evas_object_show(window);

   elm_run();
 
end:
   elm_shutdown();

   return EXIT_SUCCESS;
}
ELM_MAIN()

/* vim: set ts=8 sw=3 sts=3 expandtab cino=>5n-3f0^-2{2(0W1st0 : */
